// Copyright 2025 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/iam/v3/policy_bindings_service.proto

#include "google/cloud/iam/v3/internal/policy_bindings_metadata_decorator.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/absl_str_cat_quiet.h"
#include "google/cloud/internal/api_client_header.h"
#include "google/cloud/internal/url_encode.h"
#include "google/cloud/status_or.h"
#include <google/iam/v3/policy_bindings_service.grpc.pb.h>
#include <memory>
#include <string>
#include <utility>
#include <vector>

namespace google {
namespace cloud {
namespace iam_v3_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

PolicyBindingsMetadata::PolicyBindingsMetadata(
    std::shared_ptr<PolicyBindingsStub> child,
    std::multimap<std::string, std::string> fixed_metadata,
    std::string api_client_header)
    : child_(std::move(child)),
      fixed_metadata_(std::move(fixed_metadata)),
      api_client_header_(
          api_client_header.empty()
              ? google::cloud::internal::GeneratedLibClientHeader()
              : std::move(api_client_header)) {}

future<StatusOr<google::longrunning::Operation>>
PolicyBindingsMetadata::AsyncCreatePolicyBinding(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::iam::v3::CreatePolicyBindingRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->AsyncCreatePolicyBinding(cq, std::move(context),
                                          std::move(options), request);
}

StatusOr<google::longrunning::Operation>
PolicyBindingsMetadata::CreatePolicyBinding(
    grpc::ClientContext& context, Options options,
    google::iam::v3::CreatePolicyBindingRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->CreatePolicyBinding(context, options, request);
}

StatusOr<google::iam::v3::PolicyBinding>
PolicyBindingsMetadata::GetPolicyBinding(
    grpc::ClientContext& context, Options const& options,
    google::iam::v3::GetPolicyBindingRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetPolicyBinding(context, options, request);
}

future<StatusOr<google::longrunning::Operation>>
PolicyBindingsMetadata::AsyncUpdatePolicyBinding(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::iam::v3::UpdatePolicyBindingRequest const& request) {
  SetMetadata(
      *context, *options,
      absl::StrCat("policy_binding.name=",
                   internal::UrlEncode(request.policy_binding().name())));
  return child_->AsyncUpdatePolicyBinding(cq, std::move(context),
                                          std::move(options), request);
}

StatusOr<google::longrunning::Operation>
PolicyBindingsMetadata::UpdatePolicyBinding(
    grpc::ClientContext& context, Options options,
    google::iam::v3::UpdatePolicyBindingRequest const& request) {
  SetMetadata(
      context, options,
      absl::StrCat("policy_binding.name=",
                   internal::UrlEncode(request.policy_binding().name())));
  return child_->UpdatePolicyBinding(context, options, request);
}

future<StatusOr<google::longrunning::Operation>>
PolicyBindingsMetadata::AsyncDeletePolicyBinding(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::iam::v3::DeletePolicyBindingRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncDeletePolicyBinding(cq, std::move(context),
                                          std::move(options), request);
}

StatusOr<google::longrunning::Operation>
PolicyBindingsMetadata::DeletePolicyBinding(
    grpc::ClientContext& context, Options options,
    google::iam::v3::DeletePolicyBindingRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->DeletePolicyBinding(context, options, request);
}

StatusOr<google::iam::v3::ListPolicyBindingsResponse>
PolicyBindingsMetadata::ListPolicyBindings(
    grpc::ClientContext& context, Options const& options,
    google::iam::v3::ListPolicyBindingsRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->ListPolicyBindings(context, options, request);
}

StatusOr<google::iam::v3::SearchTargetPolicyBindingsResponse>
PolicyBindingsMetadata::SearchTargetPolicyBindings(
    grpc::ClientContext& context, Options const& options,
    google::iam::v3::SearchTargetPolicyBindingsRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("parent=", internal::UrlEncode(request.parent())));
  return child_->SearchTargetPolicyBindings(context, options, request);
}

StatusOr<google::longrunning::Operation> PolicyBindingsMetadata::GetOperation(
    grpc::ClientContext& context, Options const& options,
    google::longrunning::GetOperationRequest const& request) {
  SetMetadata(context, options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->GetOperation(context, options, request);
}

future<StatusOr<google::longrunning::Operation>>
PolicyBindingsMetadata::AsyncGetOperation(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::longrunning::GetOperationRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncGetOperation(cq, std::move(context), std::move(options),
                                   request);
}

future<Status> PolicyBindingsMetadata::AsyncCancelOperation(
    google::cloud::CompletionQueue& cq,
    std::shared_ptr<grpc::ClientContext> context,
    google::cloud::internal::ImmutableOptions options,
    google::longrunning::CancelOperationRequest const& request) {
  SetMetadata(*context, *options,
              absl::StrCat("name=", internal::UrlEncode(request.name())));
  return child_->AsyncCancelOperation(cq, std::move(context),
                                      std::move(options), request);
}

void PolicyBindingsMetadata::SetMetadata(grpc::ClientContext& context,
                                         Options const& options,
                                         std::string const& request_params) {
  context.AddMetadata("x-goog-request-params", request_params);
  SetMetadata(context, options);
}

void PolicyBindingsMetadata::SetMetadata(grpc::ClientContext& context,
                                         Options const& options) {
  google::cloud::internal::SetMetadata(context, options, fixed_metadata_,
                                       api_client_header_);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace iam_v3_internal
}  // namespace cloud
}  // namespace google
